{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "本文主要介绍几个比较实用的例子。先载入一些包：\n",
    "\n",
    "```python\n",
    "from tkinter import Tk, ttk, PhotoImage\n",
    "from tkinter import Menu, StringVar, filedialog, , Listbox \n",
    "```\n",
    "\n",
    "## 1 可绑定“动作”的按钮\n",
    "\n",
    "绑定按钮，令其输出欢迎信息：\n",
    "\n",
    "```python\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        # 创建带命令的按钮\n",
    "        self.btn = ttk.Button(self, text=\"点我!\",\n",
    "                              command=self.say_hello)\n",
    "        # 布局按钮\n",
    "        self.btn.grid(padx=120, pady=30)\n",
    "\n",
    "    def say_hello(self):\n",
    "        print(\"欢迎进入 tkinter 世界!\")\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.title(\"tkinter 的应用\")\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "效果图：\n",
    "\n",
    "![图1 带命令的按钮](https://upload-images.jianshu.io/upload_images/1114626-a3b6832be3ff72da.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 2 带图片的按钮\n",
    "\n",
    "```python\n",
    "RELIEFS = ['sunken', 'raised', 'groove', 'ridge', 'flat']\n",
    "\n",
    "\n",
    "class ButtonsApp(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        self.img = PhotoImage(file=\"python.gif\")\n",
    "        self.btn = ttk.Button(self, text=\"带图片的按钮\",\n",
    "                              image=self.img, compound='left',\n",
    "                              command=self.disable_btn)\n",
    "        self.btn.grid(row=0, column=2)\n",
    "        for i, RELIEF in enumerate(RELIEFS):\n",
    "            temp = ttk.Frame(self, relief=RELIEF,\n",
    "                             borderwidth=5, width=50, height=50)\n",
    "            label = ttk.Label(temp, text=RELIEF)\n",
    "            label.grid(row=0, column=0)\n",
    "            temp.grid(row=1, column=i, padx=10, pady=10)\n",
    "\n",
    "    def disable_btn(self):\n",
    "        self.btn.config(state='disabled')\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = ButtonsApp()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "显示效果：\n",
    "\n",
    "![图2 带图片的按钮](https://upload-images.jianshu.io/upload_images/1114626-2673fe2a30548275.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "该例子将图片放置在文字的左侧，且绑定了一个命令，该命令实现按钮的失活操作。第二行显示了不同的 relief 的 ttk.Frame。\n",
    "\n",
    "## 3 可跟踪 ttk.Entry 的“变量”\n",
    "\n",
    "```python\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        self.var = StringVar()\n",
    "        self.var.trace(\"w\", self.show_message)\n",
    "        self.entry = ttk.Entry(textvariable=self.var)\n",
    "        self.btn = ttk.Button(text=\"清除\",\n",
    "                              command=lambda: self.var.set(\"\"))\n",
    "        self.label = ttk.Label()\n",
    "        self.entry.grid()\n",
    "        self.btn.grid()\n",
    "        self.label.grid()\n",
    "\n",
    "    def show_message(self, *args):\n",
    "        value = self.var.get()\n",
    "        text = f\"你好, {value}!\" if value else ''\n",
    "        self.label.config(text=text)\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "效果图：\n",
    "\n",
    "![图3 可跟踪 ttk.Entry 的“变量”](https://upload-images.jianshu.io/upload_images/1114626-ff17c9e363cee423.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "该例子实时跟踪文本框的输入，并显示在第 3 行。\n",
    "\n",
    "## 4 验证 ttk.Entry\n",
    "\n",
    "```python\n",
    "import re\n",
    "\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        self.pattern = re.compile(\"^\\w{0,10}$\")\n",
    "        self.label = ttk.Label(text=\"输入您的用户名\")\n",
    "        vcmd = (self.register(self.validate_username), \"%i\", \"%P\")\n",
    "        self.entry = ttk.Entry(validate=\"key\",\n",
    "                              validatecommand=vcmd,\n",
    "                              invalidcommand=self.print_error)\n",
    "        self.label.pack()\n",
    "        self.entry.pack(anchor='w', padx=10, pady=10)\n",
    "\n",
    "    def validate_username(self, index, username):\n",
    "        print(\"修改 \" + index)\n",
    "        return self.pattern.match(username) is not None\n",
    "\n",
    "    def print_error(self):\n",
    "        print(\"无效的用户名\")\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "验证输入的字符是否满足条件。\n",
    "\n",
    "## 5 Listbox（列表框）\n",
    "\n",
    "该组件有一个比较重要的选项 `selectmode` 用来确定可以选择多少项，以及鼠标拖动的影响选择:\n",
    "\n",
    "- `'browse'`:通常，只能从列表框中选择一行。如果单击一个项目，然后拖动到不同的行，选择将会跟随鼠标，是默认的。\n",
    "- `'single'`：你只能选择一行，不能拖动。\n",
    "- `'multiple'`：您可以同时选择任意数量的行。点击在任意直线上，无论它是否被选中。不能拖动。\n",
    "- `'extended'`: 您可以一次选择任何相邻的多个选项。能拖动。支持Shift和Control（如Windows下的快捷键）。\n",
    "\n",
    "示例：\n",
    "\n",
    "```python\n",
    "DAYS = [\"Monday\", \"Tuesday\", \"Wednesday\", \"Thursday\",\n",
    "        \"Friday\", \"Saturday\", \"Sunday\"]\n",
    "MODES = ['single', 'browse', 'multiple', 'extended']\n",
    "\n",
    "\n",
    "class ListApp(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        self.list = Listbox()\n",
    "        self.list.insert(0, *DAYS)\n",
    "        self.print_btn = ttk.Button(self, text=\"Print selection\",\n",
    "                                    command=self.print_selection)\n",
    "        self.btns = [self.create_btn(m) for m in MODES]\n",
    "\n",
    "        self.list.pack()\n",
    "        self.print_btn.pack(fill='both')\n",
    "        for btn in self.btns:\n",
    "            btn.pack(side='left')\n",
    "\n",
    "    def create_btn(self, mode):\n",
    "        def cmd(): return self.list.config(selectmode=mode)\n",
    "        return ttk.Button(self, command=cmd,\n",
    "                          text=mode.capitalize())\n",
    "\n",
    "    def print_selection(self):\n",
    "        selection = self.list.curselection()\n",
    "        print([self.list.get(i) for i in selection])\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = ListApp()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "效果图见：\n",
    "\n",
    "![图4 列表框](https://upload-images.jianshu.io/upload_images/1114626-9d87f03a2b8fef89.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 6 鼠标触发的事件\n",
    "\n",
    "```python\n",
    "class App(ttk.Frame):\n",
    "    def __init__(self, master=None, **kw):\n",
    "        super().__init__(master, **kw)\n",
    "        style = ttk.Style()\n",
    "        style.configure(\"BG.TFrame\", foreground=\"black\", background=\"green\")\n",
    "        self.configure(style=\"BG.TFrame\", height=100, width=100)\n",
    "        self.bind_mul()\n",
    "        self.grid(padx=50, pady=50)\n",
    "\n",
    "    def bind_mul(self):\n",
    "        event_names = (\"<Button-1>\", \"<Double-Button-1>\",\n",
    "                       \"<ButtonRelease-1>\", \"<B1-Motion>\",\n",
    "                       \"<Enter>\", \"<Leave>\")\n",
    "        [self.bind(event_name, self.print_event) for event_name in event_names]\n",
    "\n",
    "    def print_event(self, event):\n",
    "        position = f\"(x={event.x}, y={event.y})\"\n",
    "        print(event.type, \"event\", position)\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    root = Tk()\n",
    "    app = App(root)\n",
    "    root.mainloop()\n",
    "```\n",
    "\n",
    "![图5 鼠标触发的事件](https://upload-images.jianshu.io/upload_images/1114626-eef057f6cb3e787c.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 7 键盘触发的事件\n",
    "\n",
    "```python\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        entry = ttk.Entry(self)\n",
    "        entry.bind(\"<FocusIn>\", self.print_type)\n",
    "        entry.bind(\"<Key>\", self.print_key)\n",
    "        entry.grid(padx=20, pady=20)\n",
    "\n",
    "    def print_type(self, event):\n",
    "        print(event.type)\n",
    "\n",
    "    def print_key(self, event):\n",
    "        print(\n",
    "            f\"Symbol: {event.keysym},Code: {event.keycode}, Char: {event.char}\")\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "效果图：\n",
    "\n",
    "![图6 键盘触发的事件](https://upload-images.jianshu.io/upload_images/1114626-a5295ac2065c507b.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 8 自定义标题栏图标\n",
    "\n",
    "```python\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        self.title(\"自定义标题栏图标\")\n",
    "        self.iconbitmap(\"python.ico\")\n",
    "        # width x height ±x±y\n",
    "        self.geometry(\"400x200+50+50\")\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "width 和 height（通常以像素为单位）非常不言自明。 “±x”（水平位置）用正负号指定，因此“+25”表示窗口的左边缘应距屏幕左边缘25个像素，而“-50”表示右边缘窗口的宽度应距屏幕右边缘50像素。 同样，“y”（垂直）位置为“+10”表示窗口的上边缘应比屏幕顶部低10个像素，而“-100”表示窗口的底部边缘应比屏幕顶部高100像素屏幕底部。\n",
    "\n",
    "![图7 自定义标题栏图标](https://upload-images.jianshu.io/upload_images/1114626-a2fa0ed382eb33bc.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 9 自定义列表框\n",
    "\n",
    "```python\n",
    "class ListFrame(ttk.Frame):\n",
    "    def __init__(self, master, items=[]):\n",
    "        super().__init__(master)\n",
    "        self.list = Listbox(self)\n",
    "        self.scroll = ttk.Scrollbar(self, orient='vertical',\n",
    "                                    command=self.list.yview)\n",
    "        self.list.config(yscrollcommand=self.scroll.set)\n",
    "        self.list.insert(0, *items)\n",
    "        self.list.pack(side='left')\n",
    "        self.scroll.pack(side='left', fill='y')\n",
    "\n",
    "    def pop_selection(self):\n",
    "        index = self.list.curselection()\n",
    "        if index:\n",
    "            value = self.list.get(index)\n",
    "            self.list.delete(index)\n",
    "            return value\n",
    "\n",
    "    def insert_item(self, item):\n",
    "        self.list.insert('end', item)\n",
    "\n",
    "\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        months = [\"January\", \"February\", \"March\", \"April\",\n",
    "                  \"May\", \"June\", \"July\", \"August\", \"September\",\n",
    "                  \"October\", \"November\", \"December\"]\n",
    "        self.frame_a = ListFrame(self, months)\n",
    "        self.frame_b = ListFrame(self)\n",
    "        self.btn_right = ttk.Button(self, text=\">\",\n",
    "                                    command=self.move_right)\n",
    "        self.btn_left = ttk.Button(self, text=\"<\",\n",
    "                                   command=self.move_left)\n",
    "\n",
    "        self.frame_a.pack(side='left', padx=10, pady=10)\n",
    "        self.frame_b.pack(side='right', padx=10, pady=10)\n",
    "        self.btn_right.pack(expand=True, ipadx=5)\n",
    "        self.btn_left.pack(expand=True, ipadx=5)\n",
    "\n",
    "    def move_right(self):\n",
    "        self.move(self.frame_a, self.frame_b)\n",
    "\n",
    "    def move_left(self):\n",
    "        self.move(self.frame_b, self.frame_a)\n",
    "\n",
    "    def move(self, frame_from, frame_to):\n",
    "        value = frame_from.pop_selection()\n",
    "        if value:\n",
    "            frame_to.insert_item(value)\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "![图8 可转移的列表框](https://upload-images.jianshu.io/upload_images/1114626-951c35c436fd377c.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 10 不同颜色的标签\n",
    "\n",
    "```python\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        colors = (\"yellow\", \"orange\", \"red\", \"green\", \"blue\")\n",
    "        labels = [ttk.Label(text=f'label_{color}', background=color)\n",
    "                  for color in colors]\n",
    "        opts = {'ipadx': 20, 'ipady': 10, 'fill': 'both'}\n",
    "        labels[0].pack(side='top', **opts)\n",
    "        labels[1].pack(side='top', **opts)\n",
    "        labels[2].pack(side='left', **opts)\n",
    "        labels[3].pack(side='left', **opts)\n",
    "        labels[4].pack(side='left', **opts)\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "效果图：\n",
    "\n",
    "![图9 不同颜色的标签](https://upload-images.jianshu.io/upload_images/1114626-09b1fae9387c4571.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "Grid 布局：\n",
    "\n",
    "```python\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        colors = (\"yellow\", \"orange\", \"red\", \"green\", \"blue\")\n",
    "        labels = [ttk.Label(text=color, background=color)\n",
    "                  for color in colors]\n",
    "\n",
    "        opts = {'ipadx': 10, 'ipady': 10, 'sticky': 'nswe'}\n",
    "        for k, lb in enumerate(labels):\n",
    "            lb.grid(row=self.set_row(k),\n",
    "                    column=self.set_column(k),\n",
    "                    rowspan=2 if k in (2, 3) else None,\n",
    "                    columnspan=3 if k == 4 else None,\n",
    "                    **opts)\n",
    "\n",
    "    def set_row(self, k):\n",
    "        if k == 1:\n",
    "            return 1\n",
    "        elif k == 4:\n",
    "            return 2\n",
    "        else:\n",
    "            return 0\n",
    "\n",
    "    def set_column(self, k):\n",
    "        if k == 2:\n",
    "            return 1\n",
    "        elif k == 3:\n",
    "            return 2\n",
    "        else:\n",
    "            return 0\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "![图10 Grid布局](https://upload-images.jianshu.io/upload_images/1114626-04db63bca80fb3e4.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "```python\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        colors = (\"yellow\", \"orange\", \"red\", \"green\", \"blue\")\n",
    "        labels = [ttk.Label(text=color, background=color)\n",
    "                  for color in colors]\n",
    "        labels[0].place(relwidth=0.25, relheight=0.25)\n",
    "        labels[1].place(x=100, anchor='n',\n",
    "                        width=100, height=50)\n",
    "        labels[2].place(relx=0.5, rely=0.5, anchor='center',\n",
    "                        relwidth=0.5, relheight=0.5)\n",
    "        labels[3].place(in_=labels[2], anchor='nw',\n",
    "                        x=2, y=2, relx=0.5, rely=0.5,\n",
    "                        relwidth=0.5, relheight=0.5)\n",
    "        labels[4].place(x=200, y=200, anchor='se',\n",
    "                        relwidth=0.25, relheight=0.25)\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "![图11 place 布局](https://upload-images.jianshu.io/upload_images/1114626-c3b121445adea7a2.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 11 创建个人信息记录表单\n",
    "\n",
    "```python\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        style = ttk.Style()\n",
    "        style.configure(\"W.TFrame\", background=\"white\")\n",
    "        group_1 = ttk.LabelFrame(self, text=\"个人信息\", style=\"W.TFrame\")\n",
    "        group_1.pack(padx=10, pady=5)\n",
    "\n",
    "        ttk.Label(group_1, text=\"性别\").grid(row=0)\n",
    "        ttk.Label(group_1, text=\"姓名\").grid(row=1)\n",
    "        ttk.Entry(group_1).grid(row=0, column=1, sticky='w')\n",
    "        ttk.Entry(group_1).grid(row=1, column=1, sticky='w')\n",
    "\n",
    "        group_2 = ttk.LabelFrame(self, text=\"地址\", style=\"W.TFrame\")\n",
    "        group_2.pack(padx=10, pady=5)\n",
    "\n",
    "        ttk.Label(group_2, text=\"国籍\").grid(row=0)\n",
    "        ttk.Label(group_2, text=\"省市\").grid(row=1)\n",
    "        ttk.Label(group_2, text=\"邮编\").grid(row=2)\n",
    "        ttk.Entry(group_2).grid(row=0, column=1, sticky='w')\n",
    "        ttk.Entry(group_2).grid(row=1, column=1, sticky='w')\n",
    "        ttk.Entry(group_2, width=8).grid(row=2, column=1,\n",
    "                                         sticky='w')\n",
    "\n",
    "        self.btn_submit = ttk.Button(self, text=\"提交\")\n",
    "        self.btn_submit.pack(padx=10, pady=10, side='right')\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "![图12 创建个人信息记录表单](https://upload-images.jianshu.io/upload_images/1114626-2503a6a1a079633b.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 12 可切换背景和前景的颜色选择器\n",
    "\n",
    "```python\n",
    "from functools import partial\n",
    "\n",
    "from tkinter.colorchooser import askcolor\n",
    "\n",
    "\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        self.title(\"Colors demo\")\n",
    "        text = \"The quick brown fox jumps over the lazy dog\"\n",
    "        self.label = ttk.Label(self, text=text)\n",
    "        self.fg_btn = ttk.Button(self, text=\"Set foreground color\",\n",
    "                                 command=partial(self.set_color, \"fg\"))\n",
    "        self.bg_btn = ttk.Button(self, text=\"Set background color\",\n",
    "                                 command=partial(self.set_color, \"bg\"))\n",
    "\n",
    "        self.label.pack(padx=20, pady=20)\n",
    "        self.fg_btn.pack(side='left', fill='both', expand=True)\n",
    "        self.bg_btn.pack(side='left', fill='both', expand=True)\n",
    "\n",
    "    def set_color(self, option):\n",
    "        color = askcolor()[1]\n",
    "        print(\"Chosen color:\", color)\n",
    "        self.label.config(**{option: color})\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "![图13 可切换背景和前景的颜色选择器](https://upload-images.jianshu.io/upload_images/1114626-88b4e03fc1f957bd.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 13 变换字体\n",
    "\n",
    "```python\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        self.title(\"Fonts demo\")\n",
    "        text = \"The quick brown fox jumps over the lazy dog\"\n",
    "        self.label = ttk.Label(text=text)\n",
    "\n",
    "        self.family = StringVar()\n",
    "        families = (\"Times\", \"Courier\", \"Helvetica\")\n",
    "        self.option = ttk.OptionMenu(self, self.family, *families)\n",
    "\n",
    "        self.size = StringVar()\n",
    "        self.size.trace(\"w\", self.set_font)\n",
    "        self.spinbox = ttk.Spinbox(from_=8, to=18,\n",
    "                                   textvariable=self.size)\n",
    "\n",
    "        self.family.set(families[0])\n",
    "        self.size.set(\"10\")\n",
    "        self.label.pack(padx=20, pady=20)\n",
    "        self.option.pack(side='left', fill='both', expand=True)\n",
    "        self.spinbox.pack(side='left', fill='both', expand=True)\n",
    "        self.family.trace(\"w\", self.set_font)\n",
    "\n",
    "    def set_font(self, *args):\n",
    "        family = self.family.get()\n",
    "        size = self.size.get()\n",
    "        self.label.config(font=(family, size))\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "![图14 可以改变字体类型与大小](https://upload-images.jianshu.io/upload_images/1114626-fcdf4a14d168fb93.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "```python\n",
    "from tkinter.font import Font\n",
    "\n",
    "\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        header = Font(family='Helvetica', size=18, weight='bold')\n",
    "        subtitle = Font(family=\"Helvetica 14 italic\")\n",
    "        style = ttk.Style()\n",
    "        style.map(\"C.TButton\",\n",
    "                  foreground=[('pressed', 'red'), ('active', 'blue')],\n",
    "                  background=[('pressed', '!disabled', 'black'), ('active', 'white')])\n",
    "        self.create_label(font=header, text=\"This is the header\")\n",
    "        self.create_label(font=subtitle, text=\"This is the subtitle\")\n",
    "        self.create_label(text=\"This is a paragraph\")\n",
    "        self.create_label(text=\"This is another paragraph\")\n",
    "        self.create_button(\"C.TButton\", text=\"See more\")\n",
    "\n",
    "    def create_label(self, **options):\n",
    "        return ttk.Label(**options).pack(padx=20, pady=5, anchor='w')\n",
    "\n",
    "    def create_button(self, style, **options):\n",
    "        return ttk.Button(style=style, **options).pack(padx=5, pady=5, anchor='e')\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "![图15 Font 设定字体](https://upload-images.jianshu.io/upload_images/1114626-626191a5858e7caf.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 14 设置鼠标光标\n",
    "\n",
    "```\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        self.title(\"Cursors demo\")\n",
    "        self.resizable(0, 0)\n",
    "        self.label = ttk.Label(self, text=\"Click the button to start\")\n",
    "        self.btn_launch = ttk.Button(self, text=\"Start!\",\n",
    "                                     command=self.perform_action)\n",
    "        self.btn_help = ttk.Button(self, text=\"Help\",\n",
    "                                   cursor=\"question_arrow\")\n",
    "\n",
    "        btn_opts = {\"side\": 'left', \"expand\": True, \"fill\": 'x',\n",
    "                    \"ipadx\": 30, \"padx\": 20, \"pady\": 5}\n",
    "        self.label.pack(pady=10)\n",
    "        self.btn_launch.pack(**btn_opts)\n",
    "        self.btn_help.pack(**btn_opts)\n",
    "\n",
    "    def perform_action(self):\n",
    "        self.btn_launch.config(state='disabled')\n",
    "        self.btn_help.config(state='disabled')\n",
    "        self.label.config(text=\"Working...\")\n",
    "        self.after(3000, self.end_action)\n",
    "        self.config(cursor=\"watch\")\n",
    "\n",
    "    def end_action(self):\n",
    "        self.btn_launch.config(state='normal')\n",
    "        self.btn_help.config(state='normal')\n",
    "        self.label.config(text=\"Done!\")\n",
    "        self.config(cursor=\"arrow\")\n",
    "\n",
    "    def set_watch_cursor(self, widget):\n",
    "        widget._old_cursor = widget.cget(\"cursor\")\n",
    "        widget.config(cursor=\"watch\")\n",
    "        for w in widget.winfo_children():\n",
    "            self.set_watch_cursor(w)\n",
    "\n",
    "    def restore_cursor(self, widget):\n",
    "        widget.config(cursor=widget.old_cursor)\n",
    "        for w in widget.winfo_children():\n",
    "            self.restore_cursor(w)\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "![图16 鼠标光标](https://upload-images.jianshu.io/upload_images/1114626-e0416b203a2f1368.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "## 15 tkinter.Text\n",
    "\n",
    "```python\n",
    "from tkinter import Text\n",
    "\n",
    "\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        self.title(\"Text demo\")\n",
    "        self.resizable(0, 0)\n",
    "        self.text = Text(width=50, height=10)\n",
    "        self.btn_clear = ttk.Button(text=\"Clear text\",\n",
    "                                    command=self.clear_text)\n",
    "        self.btn_insert = ttk.Button(text=\"Insert text\",\n",
    "                                     command=self.insert_text)\n",
    "        self.btn_print = ttk.Button(text=\"Print selection\",\n",
    "                                    command=self.print_selection)\n",
    "        self.text.pack()\n",
    "        self.btn_clear.pack(side='left', expand=True, pady=10)\n",
    "        self.btn_insert.pack(side='left', expand=True, pady=10)\n",
    "        self.btn_print.pack(side='left', expand=True, pady=10)\n",
    "\n",
    "    def clear_text(self):\n",
    "        self.text.delete(\"1.0\", 'end')\n",
    "\n",
    "    def insert_text(self):\n",
    "        self.text.insert('insert', \"Hello, world\")\n",
    "\n",
    "    def print_selection(self):\n",
    "        selection = self.text.tag_ranges('sel')\n",
    "        if selection:\n",
    "            content = self.text.get(*selection)\n",
    "            print(content)\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "![图17 Text 的示例](https://upload-images.jianshu.io/upload_images/1114626-c6d31909deb8eb11.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n",
    "\n",
    "设定超链接：\n",
    "\n",
    "```python\n",
    "import webbrowser\n",
    "from tkinter import Text\n",
    "\n",
    "class App(Tk):\n",
    "    def __init__(self):\n",
    "        super().__init__()\n",
    "        self.title(\"Text tags demo\")\n",
    "        self.text = Text(self, width=50, height=10)\n",
    "        self.btn_link = ttk.Button(self, text=\"Add hyperlink\",\n",
    "                                   command=self.add_hyperlink)\n",
    "\n",
    "        self.text.tag_config(\"link\", foreground=\"blue\", underline=1)\n",
    "        self.text.tag_bind(\"link\", \"<Button-1>\", self.open_link)\n",
    "        self.text.tag_bind(\"link\", \"<Enter>\",\n",
    "                           lambda _: self.text.config(cursor=\"hand2\"))\n",
    "        self.text.tag_bind(\"link\", \"<Leave>\",\n",
    "                           lambda e: self.text.config(cursor=\"\"))\n",
    "\n",
    "        self.text.pack()\n",
    "        self.btn_link.pack(expand=True)\n",
    "\n",
    "    def add_hyperlink(self):\n",
    "        selection = self.text.tag_ranges('sel')\n",
    "        if selection:\n",
    "            self.text.tag_add(\"link\", *selection)\n",
    "\n",
    "    def open_link(self, event):\n",
    "        position = f\"@{event.x},{event.y} + 1c\"\n",
    "        index = self.text.index(position)\n",
    "        prevrange = self.text.tag_prevrange(\"link\", index)\n",
    "        url = self.text.get(*prevrange)\n",
    "        webbrowser.open(url)\n",
    "\n",
    "\n",
    "if __name__ == \"__main__\":\n",
    "    app = App()\n",
    "    app.mainloop()\n",
    "```\n",
    "\n",
    "![图18 设定超链接](https://upload-images.jianshu.io/upload_images/1114626-2ec90b717ca14f06.png?imageMogr2/auto-orient/strip%7CimageView2/2/w/1240)\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": []
  }
 ],
 "metadata": {
  "language_info": {
   "name": "python"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
